@@ -20,10 +20,9 @@ from nebula2.fbthrift.transport import TSocket, TTransport
 from nebula2.fbthrift.transport.TTransport import TTransportException
 from nebula2.fbthrift.protocol import TBinaryProtocol
 
-from nebula2.graph import (
-    ttypes,
-    GraphService
-)
+from nebula2.common.ttypes import ErrorCode
+
+from nebula2.graph import GraphService
 
 from nebula2.Exception import (
     AuthFailedException,
@@ -37,53 +36,77 @@ from nebula2.data.ResultSet import ResultSet
 logging.basicConfig(level=logging.INFO, format='[%(asctime)s]:%(message)s')
 
 
+class AuthResult(object):
+    def __init__(self, session_id, timezone_offset, timezone_name):
+        self._session_id = session_id
+        self._timezone_offset = timezone_offset
+        self._timezone_name = timezone_name
+
+    def get_session_id(self):
+        return self._session_id
+
+    def get_timezone_offset(self):
+        return self._timezone_offset
+
+    def get_timezone_name(self):
+        return self._timezone_name
+
+
 class Session(object):
-    def __init__(self, connection, session_id, pool, retry_connect=True):
-        self.session_id = session_id
+    def __init__(self, connection, auth_result: AuthResult, pool, retry_connect=True):
+        self._session_id = auth_result.get_session_id()
+        self._timezone_offset = auth_result.get_timezone_offset()
         self._connection = connection
         self._timezone = 0
         self._pool = pool
         self._retry_connect = retry_connect
 
     def execute(self, stmt):
-        """
-        execute statement
+        """execute statement
+
         :param stmt: the ngql
         :return: ResultSet
         """
         if self._connection is None:
             raise RuntimeError('The session has released')
         try:
-            return ResultSet(self._connection.execute(self.session_id, stmt))
+            start_time = time.time()
+            resp = self._connection.execute(self._session_id, stmt)
+            end_time = time.time()
+            return ResultSet(resp,
+                             all_latency=int((end_time - start_time) * 1000000),
+                             timezone_offset=self._timezone_offset)
         except IOErrorException as ie:
             if ie.type == IOErrorException.E_CONNECT_BROKEN:
                 self._pool.update_servers_status()
                 if self._retry_connect:
                     if not self._reconnect():
                         logging.warning('Retry connect failed')
-                        raise IOErrorException(IOErrorException.E_ALL_BROKEN, 'All connections are broken')
-                    try:
-                        return ResultSet(self._connection.execute(self.session_id, stmt))
-                    except Exception:
-                        raise
+                        raise IOErrorException(IOErrorException.E_ALL_BROKEN, ie.message)
+                    resp = self._connection.execute(self._session_id, stmt)
+                    end_time = time.time()
+                    return ResultSet(resp,
+                                     all_latency=int((end_time - start_time) * 1000000),
+                                     timezone_offset=self._timezone_offset)
             raise
         except Exception:
             raise
 
     def release(self):
-        """
-        release the connection to pool
+        """release the connection to pool, and the session couldn't been use again
+
+        :return:
         """
         if self._connection is None:
             return
-        self._connection.signout(self.session_id)
+        self._connection.signout(self._session_id)
         self._connection.is_used = False
         self._connection = None
 
     def ping(self):
-        """
-        check the connection is ok
-        :return Boolean
+        """check the connection is ok
+
+        :return: True or False
         """
         if self._connection is None:
             return False
@@ -91,6 +114,7 @@ class Session(object):
 
     def _reconnect(self):
         try:
+            self._connection.is_used = False
             conn = self._pool.get_connection()
             if conn is None:
                 return False
@@ -119,17 +143,16 @@ class ConnectionPool(object):
         self._configs = None
         self._lock = RLock()
         self._pos = -1
-        self._check_delay = 5 * 60  # unit seconds
         self._close = False
 
     def __del__(self):
         self.close()
 
     def init(self, addresses, configs):
-        """
-        init the connection pool
+        """init the connection pool
+
         :param addresses: the graphd servers' addresses
-        :param configs: the config
+        :param configs: the config of the pool
         :return: if all addresses are ok, return True else return False.
         """
         if self._close:
@@ -147,6 +170,8 @@ class ConnectionPool(object):
                 self._addresses_status[ip_port] = self.S_BAD
                 self._connections[ip_port] = deque()
 
+        self.update_servers_status()
+
         # detect the services
         self._period_detect()
 
@@ -164,19 +189,19 @@ class ConnectionPool(object):
         return True
 
     def get_session(self, user_name, password, retry_connect=True):
-        """
-        get session
-        :param user_name:
-        :param password:
-        :param retry_connect: if auto retry connect
-        :return: void
+        """get session
+
+        :param user_name: the user name to authenticate graphd
+        :param password: the password to authenticate graphd
+        :param retry_connect:
+        :return: Session
         """
         connection = self.get_connection()
         if connection is None:
             raise NotValidConnectionException()
         try:
-            session_id = connection.authenticate(user_name, password)
-            return Session(connection, session_id, self, retry_connect)
+            auth_result = connection.authenticate(user_name, password)
+            return Session(connection, auth_result, self, retry_connect)
         except Exception:
             raise
 
@@ -189,8 +214,8 @@ class ConnectionPool(object):
 
         When session_context is exited, the connection will be released.
 
-        :param user_name:
-        :param password:
+        :param user_name: the user name to authenticate graphd
+        :param password: the password to authenticate graphd
         :param retry_connect: if auto retry connect
         :return: contextlib._GeneratorContextManager
         """
@@ -205,9 +230,9 @@ class ConnectionPool(object):
                 session.release()
 
     def get_connection(self):
-        """
-        get available connection
-        :return: Connection Object
+        """get available connection
+
+        :return: Connection
         """
         with self._lock:
             if self._close:
@@ -246,13 +271,11 @@ class ConnectionPool(object):
                 return None
             except Exception as ex:
                 logging.error('Get connection failed: {}'.format(ex))
-                import traceback
-                print(traceback.format_exc())
                 return None
 
     def ping(self, address):
-        """
-        check the server is ok
+        """check the server is ok
+
         :param address: the server address want to connect
         :return: True or False
         """
@@ -266,8 +289,8 @@ class ConnectionPool(object):
             return False
 
     def close(self):
-        """
-        close all connections in pool
+        """close all connections in pool
+
         :return: void
         """
         with self._lock:
@@ -279,9 +302,9 @@ class ConnectionPool(object):
             self._close = True
 
     def connnects(self):
-        """
-        get the number of existing connections
-        :return: int
+        """get the number of existing connections
+
+        :return: the number of connections
         """
         with self._lock:
             count = 0
@@ -290,8 +313,8 @@ class ConnectionPool(object):
             return count
 
     def in_used_connects(self):
-        """
-        get the number of the used connections
+        """get the number of the used connections
+
         :return: int
         """
         with self._lock:
@@ -303,8 +326,8 @@ class ConnectionPool(object):
             return count
 
     def get_ok_servers_num(self):
-        """
-        get the number of the ok servers
+        """get the number of the ok servers
+
         :return: int
         """
         count = 0
@@ -323,8 +346,7 @@ class ConnectionPool(object):
         return ', '.join(msg_list)
 
     def update_servers_status(self):
-        """
-        update the servers' status
+        """update the servers' status
         """
         for address in self._addresses:
             if self.ping(address):
@@ -338,7 +360,7 @@ class ConnectionPool(object):
         with self._lock:
             for addr in self._connections.keys():
                 conns = self._connections[addr]
-                for connection in conns:
+                for connection in list(conns):
                     if not connection.is_used:
                         if not connection.ping():
                             logging.debug('Remove the not unusable connection to {}'.format(connection.get_address()))
@@ -349,11 +371,11 @@ class ConnectionPool(object):
                             conns.remove(connection)
 
     def _period_detect(self):
-        if self._close:
+        if self._close or self._configs.interval_check < 0:
             return
         self.update_servers_status()
         self._remove_idle_unusable_connection()
-        timer = threading.Timer(self._check_delay, self._period_detect)
+        timer = threading.Timer(self._configs.interval_check, self._period_detect)
         timer.setDaemon(True)
         timer.start()
 
@@ -363,13 +385,22 @@ class Connection(object):
 
     def __init__(self):
         self._connection = None
-        self.start_use_time = 0
+        self.start_use_time = time.time()
         self._ip = None
         self._port = None
+        self._timeout = 0
 
     def open(self, ip, port, timeout):
+        """open the connection
+
+        :param ip: the server ip
+        :param port: the server port
+        :param timeout: the timeout for connect and execute
+        :return: void
+        """
         self._ip = ip
         self._port = port
+        self._timeout = timeout
         try:
             s = TSocket.TSocket(self._ip, self._port)
             if timeout > 0:
@@ -381,27 +412,62 @@ class Connection(object):
         except Exception:
             raise
 
+    def _reopen(self):
+        """reopen the connection
+
+        :return:
+        """
+        self.close()
+        self.open(self._ip, self._port, self._timeout)
+
     def authenticate(self, user_name, password):
+        """authenticate to graphd
+
+        :param user_name: the user name
+        :param password: the password
+        :return:
+        """
         try:
             resp = self._connection.authenticate(user_name, password)
-            if resp.error_code != ttypes.ErrorCode.SUCCEEDED:
+            if resp.error_code != ErrorCode.SUCCEEDED:
                 raise AuthFailedException(resp.error_msg)
-            return resp.session_id
+            return AuthResult(resp.session_id, resp.time_zone_offset_seconds, resp.time_zone_name)
         except TTransportException as te:
+            if te.message.find("timed out"):
+                self._reopen()
             if te.type == TTransportException.END_OF_FILE:
                 self.close()
             raise IOErrorException(IOErrorException.E_CONNECT_BROKEN, te.message)
 
     def execute(self, session_id, stmt):
+        """execute interface with session_id and ngql
+
+        :param session_id: the session id get from result of authenticate interface
+        :param stmt: the ngql
+        :return: ExecutionResponse
+        """
         try:
             resp = self._connection.execute(session_id, stmt)
             return resp
-        except TTransportException as te:
-            if te.type == TTransportException.END_OF_FILE:
-                self.close()
-            raise IOErrorException(IOErrorException.E_CONNECT_BROKEN, te.message)
+        except Exception as te:
+            if isinstance(te, TTransportException):
+                if te.message.find("timed out") > 0:
+                    self._reopen()
+                    raise IOErrorException(IOErrorException.E_TIMEOUT, te.message)
+                elif te.type == TTransportException.END_OF_FILE:
+                    raise IOErrorException(IOErrorException.E_CONNECT_BROKEN, te.message)
+                elif te.type == TTransportException.NOT_OPEN:
+                    raise IOErrorException(IOErrorException.E_NOT_OPEN, te.message)
+                else:
+                    raise IOErrorException(IOErrorException.E_UNKNOWN, te.message);
+            raise
 
     def signout(self, session_id):
+        """tells the graphd can release the session info
+
+        :param session_id:the session id
+        :return: void
+        """
         try:
             self._connection.signout(session_id)
         except TTransportException as te:
@@ -409,7 +475,7 @@ class Connection(object):
                 self.close()
 
     def close(self):
-        """
+        """close the connection
 
         :return: void
         """
@@ -419,25 +485,34 @@ class Connection(object):
             logging.error('Close connection to {}:{} failed:{}'.format(self._ip, self._port, e))
 
     def ping(self):
-        """
-        check the connection if ok
-        :return: Boolean
+        """check the connection if ok
+        :return: True or False
         """
         try:
-            self._connection.execute(0, 'YIELD 1;')
+            resp = self._connection.execute(0, 'YIELD 1;')
             return True
-        except TTransportException as te:
-            if te.type == TTransportException.END_OF_FILE:
-                return False
-        return True
+        except Exception:
+            return False
 
     def reset(self):
+        """reset the idletime
+
+        :return: void
+        """
         self.start_use_time = time.time()
 
     def idle_time(self):
-        if not self.is_used:
+        """get idletime of connection
+
+        :return: idletime
+        """
+        if self.is_used:
             return 0
-        return time.time() - self.start_use_time
+        return (time.time() - self.start_use_time) * 1000
 
     def get_address(self):
+        """get the address of the connected service
+
+        :return: (ip, port)
+        """
         return (self._ip, self._port)
